/**
 * Mixer controller responsible for communication between UI and Models/Services
 *
 * soundsFactory - module responsible for loading all sounds and handling switching from one preset to the next
 * userStorage - module holding information about user action throughout website. List of presets, User data
 * platformType - determines wheter we're on mobile on desktop device, based on MobileDetect js implementation
 * $scope - standard Angular variable that determies scope of controller.
 */

controllers.controller('mixerController', ["$scope", "soundsFactory", "userStorage", "platformType", function ($scope, soundsFactory, userStorage, platformType) {

    $scope.soundsProgress = 0;
    $scope.soundsLoaded = false;
    $scope.isMixing = false; //flag that determines if we're mixing something
    $scope.tracksPlaying = "tracks-0"; //default class name assigned via data binding in html
    $scope.presetSelected = -1;
    $scope.isMobile = platformType.isMobile;
    $scope.platformType = platformType;
    $scope.baseGif = -1;

    $scope.$on(Events.Sounds.SOUNDS_LOADED, function () {
        //render UI and show presets
        $scope.manifest = soundsFactory.getLoadedSounds();
        $scope.soundsLoaded = true;

        //tell elements to change state to fixed
        var positionFixed = false;
        if( !platformType.isPhone ){
            positionFixed = true;
        }

        $scope.$emit(Events.Utils.POSITION_FIXED, positionFixed);

        // trigger muted playback only on desktop platforms, as on mobile trigger by click must be done
        if (!$scope.isMobile) {
            //start playing sounds
            soundsFactory.play();
            soundsFactory.launchPreset(4);//launch vocal once sounds are loaded
        }

        $scope.$evalAsync();// use $evalAsync() as this event is not coming from Angular
    });

    /**
     * SOUND_STATE_CHANGE event determines that user interacted with mixer hence we should update
     * visible gifs.
     *
     * $scope.baseGif variable determines which one of "base" gifs should be visible. ie. when we see vocal
     * and launch drums, we want vocal to be substitutted by drums, but when we add guitar, we want to add gif to grid,
     * instead of subsituting currently playing one
     */

    $scope.$on(Events.Sounds.SOUND_STATE_CHANGE, function (event, numOfSoundsPlaying) {

        $scope.baseGif = -1;

        var gifsVisible = numOfSoundsPlaying;

        angular.forEach(soundsFactory.getPlayingTracks(), function (soundIndex, index) {
            switch (soundIndex) {
                case 0:
                case 1:
                    $scope.baseGif = 1;
                    gifsVisible--;
                    break;
                case 4:
                case 5:
                case 6:
                case 7:
                    $scope.baseGif = 2;
                    gifsVisible--;
                    break;
                case 13:
                    $scope.baseGif = 3;
                    gifsVisible--;
                    break;
            }
        });

        //determines css class that sets looped gifs in html flex container
        $scope.tracksPlaying = "tracks-" + (gifsVisible + 1);

    });

    $scope.$on(Events.Sounds.SOUND_LOAD_PROGRESS, function (event, data) {
        $scope.soundsProgress = parseInt(data * 100);
        $scope.$evalAsync();//digest - this event is triggered by 3rd party library
    });

    $scope.soundClick = function (group, howl) {

        if (!soundsFactory.isPlaying() && platformType.isPhone ) {
            //for mobile phones, as we don't have preset screen there
            $scope.isMixing = true;
            soundsFactory.play();
        }

        if (soundsFactory.getPlayingTracks().length === 0 && platformType.isMobile) {
            //console.log( soundsFactory.getPlayingTracks() , $scope.isMobile );
            soundsFactory.launchPreset(4);//force to launch vocal on mobile
            //$scope.isMixing = false;
        }

        if( $scope.isMixing ){
            soundsFactory.nextSample(group, howl);
        }
    };

    $scope.launchPreset = function (index) {
        soundsFactory.launchPreset(index);
    };

    $scope.resetPresets = function () {
        if (!$scope.isMixing) {
            soundsFactory.launchPreset(-1);
        }
    };

    $scope.startMixing = function (launchSilent, presetNumber) {

        //check if mobile device
        if ($scope.isMobile) {

            //check if sound mixer is already initiated
            if (!soundsFactory.isPlaying()) {
                soundsFactory.play();
            }

            if ($scope.presetSelected === presetNumber) {
                //executed if selected preset was clicked second time
                $scope.isMixing = true;
            } else {
                //set selected preset number
                $scope.presetSelected = presetNumber;
            }

        } else {
            $scope.isMixing = true;
        }
    };

    $scope.$on(Events.Navigation.SET_STAGE, function (event, stage) {
        $scope.isMixing = false;
    });

    $scope.goHome = function () {
        $scope.$emit(Events.Navigation.SET_STAGE, Events.Sections.HOMEPAGE);
    };

    $scope.resetMixing = function () {
        $scope.isMixing = false;
        $scope.resetPresets();
    };

    $scope.sendMix = function () {
        userStorage.storeRemix(soundsFactory.getPlayingTracks('manifestIndex'));
        $scope.$emit(Events.Navigation.SET_STAGE, Events.Sections.FORM);
    };

    soundsFactory.loadSounds();

}]);